# Copyright (C) 2018 King Paul

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version (the "AGPL-3.0+").

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License and the additional terms for more
# details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# ADDITIONAL TERMS are also included as allowed by Section 7 of the GNU
# Affero General Public License. These additional terms are Sections 1, 5,
# 6, 7, 8, and 9 from the Apache License, Version 2.0 (the "Apache-2.0")
# where all references to the definition "License" are instead defined to
# mean the AGPL-3.0+.

# You should have received a copy of the Apache-2.0 along with this
# program. If not, see <http://www.apache.org/licenses/LICENSE-2.0>.

"""@author: king.r.paul@gmail.com
"""


import csv
from string import ascii_letters as LETTERS
from string import digits as DIGITS
from os.path import dirname, join

from matplotlib.gridspec import GridSpec
from matplotlib import pyplot as plt
import numpy as np

from .sinogram import unshuffle, crop


def tomo_sinogram_csv2pdf(file_name='./sinogram.csv', show=True, save=True):
    """
    Convert a CSV sinogram file into a PDF fluence map collection, by
    unshuffling the sinogram, i.e. separating leaf pattern into
    the 51 tomtherapy discretization angles; display & save result.

    Keyword Args:
        file_name:
            path to csv sinogram file, file formatted as follows
                "Patient name: ANONYMOUS^PATIENT, ID: 00000",,,,,,,,,
                ,0,0,0,0,0,0,0,0,0,0,0,0,,0.39123373,0.366435635 ...
            Demographics on row1, with each following row corresponding
            to a single couch step increment and comprised of 64 cells.
            Each cell in a row corresponding to an mlc leaf, and
            containing its leaf-open fraction.
            This format is produced by ExportTomoSinogram.py, shared by
            Brandon Merz on on the RaySearch customer forum, 1/18/2018.
        show: diplay on screen
        save: save to disk
    """

    fig = plt.figure(figsize=(7.5, 11))
    grid_spec = GridSpec(nrows=9, ncols=6, hspace=None, wspace=None,
                         left=0.05, right=0.9, bottom=0.02, top=0.975)

    with open(file_name, 'r') as csvfile:

        # PATIENT NAME & ID
        pat_name, pat_num = csvfile.readline().split('ID:')
        pat_name = pat_name.replace('Patient name:', '')
        pat_name_last, pat_name_first = pat_name.split('^')

        pat_name_last = ''.join([c for c in pat_name_last if c in LETTERS])
        pat_name_first = ''.join([c for c in pat_name_first if c in LETTERS])
        pat_num = ''.join([c for c in pat_num if c in DIGITS])

        document_id = pat_num + ' - ' + pat_name_last + ', ' + pat_name_first

        # SINOGRAM
        reader = csv.reader(csvfile, delimiter=',')
        array = np.asarray([line[1:] for line in reader]).astype(float)

    result = unshuffle(crop(array))

    fig.text(0.03, 0.985, document_id,
             horizontalalignment='left', verticalalignment='center')

    for idx, angle in enumerate(result):
        subplot = fig.add_subplot(grid_spec[idx])
        _ = subplot.imshow(angle, cmap='gray')
        subplot.axes.get_xaxis().set_visible(False)
        subplot.axes.get_yaxis().set_visible(False)
        subplot.set_title('{0:.0f} dg'.format(7.06*idx), fontsize=9)

    if save:
        plt.savefig(join(dirname(file_name), document_id + ' Sinogram.pdf'))

    if show:
        plt.show()


if __name__ == '__main__':
    import os

    try:
        test = os.path.join(os.getcwd(), 'src', 'pymedphys',
                            '_labs', 'paulking', 'sinogram.csv')
        tomo_sinogram_csv2pdf(test, show=True, save=True)
    except IOError:
        print('No sinogram csv file.')



# Copyright (C) 2019 Paul King

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version (the "AGPL-3.0+").

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License and the additional terms for more
# details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# ADDITIONAL TERMS are also included as allowed by Section 7 of the GNU
# Affero General Public License. These additional terms are Sections 1, 5,
# 6, 7, 8, and 9 from the Apache License, Version 2.0 (the "Apache-2.0")
# where all references to the definition "License" are instead defined to
# mean the AGPL-3.0+.

# You should have received a copy of the Apache-2.0 along with this
# program. If not, see <http://www.apache.org/licenses/LICENSE-2.0>.

from PIL import Image

import numpy as np
import matplotlib.image as mpimg


def read_narrow_png(file_name, step_size=0.1):
    """  Extract a an relative-density profilee from a narrow png file.

    Source file is a full color PNG that is sufficiently narrow that
    density uniform along its short dimension. The image density along
    its long dimension is reflective of a dose distribution. Requires
    Python PIL.

    Parameters
    ----------
    file_name : str
    step-size : float, optional
        Distance output increment in cm, defaults to 1 mm

    Returns
    -------
    array_like
        Image profile as a list of (distance, density) tuples, where density
        is an average color intensity value as represented in the source file.

    Raises
    ------
    ValueError
        If image is not narrow, i.e. aspect ratio <= 5
    AssertionError
        If step_size is too small, i.e. step_size <= 12.7 / dpi

    """

    image_file = Image.open(file_name)
    assert image_file.mode == 'RGB'
    dpi_horiz, dpi_vert = image_file.info['dpi']

    image_array = mpimg.imread(file_name)

    # DIMENSIONS TO AVG ACROSS DIFFERENT FOR HORIZ VS VERT IMG
    if image_array.shape[0] > 5*image_array.shape[1]:    # VERT
        image_vector = np.average(image_array, axis=(1, 2))
        pixel_size_in_cm = (2.54 / dpi_vert)
    elif image_array.shape[1] > 5*image_array.shape[0]:  # HORIZ
        image_vector = np.average(image_array, axis=(0, 2))
        pixel_size_in_cm = (2.54 / dpi_horiz)
    else:
        raise ValueError('The PNG file is not a narrow strip.')
    assert step_size > 5 * pixel_size_in_cm, "step size too small"

    if image_vector.shape[0] % 2 == 0:
        image_vector = image_vector[:-1]  # SO ZERO DISTANCE IS MID-PIXEL

    length_in_cm = image_vector.shape[0] * pixel_size_in_cm
    full_resolution_distances = np.arange(-length_in_cm/2,
                                          length_in_cm/2,
                                          pixel_size_in_cm)

    # TO MOVE FROM FILM RESOLUTION TO DESIRED PROFILE RESOLUTION
    num_pixels_to_avg_over = int(step_size/pixel_size_in_cm)
    sample_indices = np.arange(num_pixels_to_avg_over/2,
                               len(full_resolution_distances),
                               num_pixels_to_avg_over).astype(int)
    downsampled_distances = list(full_resolution_distances[sample_indices])

    downsampled_density = []
    for idx in sample_indices:  # AVERAGE OVER THE SAMPLING WINDOW
        avg_density = np.average(
            image_vector[int(idx - num_pixels_to_avg_over / 2):
                         int(idx + num_pixels_to_avg_over / 2)])
        downsampled_density.append(avg_density)

    zipped_profile = list(zip(downsampled_distances, downsampled_density))
    return zipped_profile



# Copyright (C) 2019 Paul King, Simon Biggs

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version (the "AGPL-3.0+").

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License and the additional terms for more
# details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# ADDITIONAL TERMS are also included as allowed by Section 7 of the GNU
# Affero General Public License. These additional terms are Sections 1, 5,
# 6, 7, 8, and 9 from the Apache License, Version 2.0 (the "Apache-2.0")
# where all references to the definition "License" are instead defined to
# mean the AGPL-3.0+.

# You should have received a copy of the Apache-2.0 along with this
# program. If not, see <http://www.apache.org/licenses/LICENSE-2.0>.


from copy import deepcopy
from typing import Callable

import numpy as np
from scipy import interpolate
import matplotlib.pyplot as plt

import xarray as xr

# from .._level1.coreobjects import _PyMedPhysBase


NumpyFunction = Callable[[np.ndarray], np.ndarray]


# pylint: disable = C0103, C0121


class DoseBase():
    def __init__(self, data, coords=None, dims=None):
        self.new_xarray(data, coords=coords, dims=dims)

    def new_xarray(self, data, coords=None, dims=None):
        self._xarray = xr.DataArray(data, coords, dims, name='dose')

    @property
    def data(self) -> np.ndarray:
        return self._xarray.data  # type: ignore

    @data.setter
    def data(self, array) -> None:
        array = np.array(array)
        self._xarray.data = array

    def to_xarray(self):
        return deepcopy(self._xarray)

    def to_pandas(self):
        return self.to_xarray().to_pandas()

    def to_dict(self):
        return self.to_xarray().to_dict()

    def deepcopy(self):
        return deepcopy(self)


class Dose1D(DoseBase):
    def __init__(self, x, data):
        self.new_xarray(x, data)

    def new_xarray(self, x, data):
        coords = [('x', x)]
        super().new_xarray(data, coords=coords)

    @property
    def x(self) -> np.ndarray:
        return self._xarray.x.data  # type: ignore

    @x.setter
    def x(self, array) -> None:
        array = np.array(array)
        if len(np.shape(array)) != 1:
            raise ValueError("`x` must be of one dimension.")

        self._xarray.x.data = array

    @property
    def interp(self) -> NumpyFunction:
        return interpolate.interp1d(self.x, self.data)  # type: ignore

    def shift(self, applied_shift, inplace=False):
        if inplace:
            adjusted_object = self
        else:
            adjusted_object = self.deepcopy()

        adjusted_object.x = adjusted_object.x + applied_shift

        if not inplace:
            return adjusted_object

    def plot(self):
        return plt.plot(self.x, self.data, 'o-')

    def interactive(self):
        pass


class DoseProfile(Dose1D):
    def interactive(self):
        pass

    def resample(self, new_x):
        resampled_profile = self.deepcopy()

        new_data = resampled_profile.interp(new_x)
        resampled_profile.new_xarray(new_x, new_data)

        return resampled_profile

    def dose_normalise(self):
        pass

    def dist_normalise(self):
        pass

    @property
    def umbra(self):
        pass

    @property
    def edges(self):
        pass

    @property
    def centre(self):
        pass

    def shift_to_centre(self):
        pass

    @property
    def flatness(self):
        pass

    @property
    def symmetry(self):
        pass

    def symmetrise(self):
        pass


class DoseDepth(Dose1D):
    pass



        # print(centre.get())

        # profile = Profile().from_pulse(centre.get(), width.get(),
        #                                (start.get(), end.get()), increment.get())
        # self.subplot.plot(profile.x, profile.y)
        # self.canvas.draw()



# Copyright (C) 2018 Paul King

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version (the "AGPL-3.0+").

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License and the additional terms for more
# details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# ADDITIONAL TERMS are also included as allowed by Section 7 of the GNU
# Affero General Public License. These additional terms are Sections 1, 5,
# 6, 7, 8, and 9 from the Apache License, Version 2.0 (the "Apache-2.0")
# where all references to the definition "License" are instead defined to
# mean the AGPL-3.0+.

# You should have received a copy of the Apache-2.0 along with this
# program. If not, see <http://www.apache.org/licenses/LICENSE-2.0>.

from collections import namedtuple

import numpy as np
from scipy.interpolate import interp1d


def read_prs(file_name):
    """
    Read native Profiler data file and return dose profiles.

    Parameters
    ----------
    file_name : string
        | file name of Profiler file including path

    Returns
    -------
    Profiler : named tuple
        | Profiler.cax = float dose at central axis
        | Profiler.x = list of (x, dose) tuples
        | Profiler.y = list of (y, dose) tuples
    """

    Profiler = namedtuple('Profiler', ['cax', 'x', 'y'])

    with open(file_name) as profiler_file:
        for row in profiler_file.readlines():
            contents = row
            if contents[:11] == "Calibration" and "File" not in contents:
                calibs = np.array(contents.split())[1:].astype(float)
            elif contents[:5] == "Data:":
                counts = np.array(contents.split()[5:145]).astype(float)
            elif contents[:15] == "Dose Per Count:":
                dose_per_count = (float(contents.split()[-1]))
        assert (len(calibs)) == (len(counts)) == 140
        assert dose_per_count > 0.0
    dose = counts * dose_per_count * calibs

    y_vals = [-16.4 + 0.4*i for i in range(83)]
    x_vals = [-11.2 + 0.4*i for i in range(57)]

    x_prof = list(zip(y_vals, dose[:57]))
    y_prof = list(zip(x_vals, dose[57:]))

    assert np.allclose(y_prof[41][1], x_prof[28][1])
    cax_dose = y_prof[41][1]

    return Profiler(cax_dose, x_prof, y_prof)


def write_prs(x_prof, y_prof,file_name=None,
              version=7,
              meas_date='01/01/1970', meas_time='00:00:01',
              descrip='', instit='', file_cal="NONE", collector='Profiler2',
              coll_sn='0000000', coll_rev='C',
              f_ware_ver='0.0.0', s_ware_v='0.0.0.0',
              depth='1.0', nom_gain='1', orient='Sagittal', ssd='100',
              bm_mode='Pulsed', tray='No', align='None', interval='50',
              room='', mach_type='', mach_mod='',mach_num='',
              bm_modal='Photon', bm_energy='0', jaws=('5','5','5','5'),
              wdg_type='None', wdg_ang='0', rate='0', dose='0',
              gantry='180', collimator='180', bkg_used='true',
              pulse_mode='true', alz_panels='0000000', cal_sn='0000000',
              cal_rev='A', temp='-100', dose_cal = '0.001', d_at_cal='100.0',
              abs_cal='false', energy_cal='0', cal_date='01/01/1970',
              cal_time='00:00:01', cal_cmnts='', m_frame='false',
              updates='0', pulses='0', time='0.0',
              detectors=('83', '57', '0', '4'), spacing='0.4',
              concat='false'):

    """
    Write dose profiles to a file in the native Profiler data format.

    Arguments:
        x_prof -- x profile, i.e. [(distance, dose), ...]
        y_prof -- y profile, i.e. [(distance, dose), ...]

    Keyword Argument [generally needed]:
        file_name -- output long file name, None -> return string only

    Keyword Arguments [not generally needed]:
        ** defaults to reasonable values
        ** many of these values do not affect converted profiles
        ---------
        meas_date -- string, e.g. '11/10/2018'
        meas_time -- string, e.g. '16:56:28'
        descrip   -- string description
        instit    -- string institution
        file_cal  -- long path to calibraiton file, N/A for converted
        collector -- string collector model, i.e. Profiler2
        coll_sn   -- string collector serial number, e.g. '1234567'
        coll_rev  -- string collector revision letter, e.g. 'C'
        f_ware_ver-- string firmware version, e.g. '1.2.1'
        s_ware_v  -- string software version, e.g. '1.1.0.0'
        depth     -- string measurement depth in cm
        nom_gain  -- string nominal gain, e.g. '1',
        orient    -- string orientation, e.g. 'Sagittal',
        ssd       -- string SSD in cm, e.g. '100'
        bm_mode   -- string beam mode, e.g. 'Pulsed'
        tray      -- string e.g. 'No', 'Yes'
        align     -- string, e.g. 'None', 'Light Field', 'Cross Hair'
        interval  -- string collection interval, e.g. 50, must be >0
        room      -- string room description
        mach_type -- string machine type description
        mach_mod  -- string machine model
        mach_num  -- string machine serial number
        bm_modal  -- string beam modality, e.g.'Photon', 'Electron', 'Cobalt'
        bm_energy -- string beam nominal in MV or MeV, e.g. '6'
        jaws      -- tuple of string jaw positions in cm: (l, r, t, b)
        wdg_type  -- string type, e.g. 'None', 'Static', 'Dynamic', 'Virtual'
        wdg_ang   -- string angle, e.g. '0',
        rate      -- string dose rate, e.g. '300'
        dose      -- string total dose, e.g. '100'
        gantry    -- string gantry angle in deg, e.g. '180'
        collimator-- string collimator angle in deg, e.g. '180'
        bkg_used  -- string background used, e.g. 'true'
        pulse_mode-- string pulse mode, e.g.'true'
        alz_panels-- string analyze panels, e.g. ='1015679')
        cal_sn    -- string calibration serial num, e.g. '1234567'
        cal_rev   -- string calibration revision letter, e.g. 'C',
        temp      -- string temparature, internal scale, e.g. '-98.96726775'
        dose_cal  -- string dose per count, e.g. '0.001' for 1000 cts per cGy
        d_at_cal  -- string dose used for absolute calibration, e.g. "75.40153"
        abs_cal   -- string absolute calibration, e.g. 'false', 'true'
        energy_cal-- string energy of calibration, e.g. '4'
        cal_date  -- string, e.g. '11/10/2018'
        cal_time  -- string, e.g. '16:56:28'
        cal_cmnts -- string calibration comments
        m_frame   -- string mult-frame, e.g.'false', 'true
        updates   -- string number of updates, e.g.'25',
        pulses    -- string number of pulses, e.g.'3939'
        time      -- string total measurement time in sec, e.g. '12.5'
        detectors -- tuple of string, Profiler2 -> ('83', '57', '0', '4')
        spacing   -- string detector spacing, e.g.'0.4'
        concat    -- string concatenated measuremets, e.g. 'false'


    Returns:
        The contents of the file as a list of strings.
    """

    # EXTEND PROFILE TAILS, TO ENSURE LONGER THAN DEVICE
    x_prof = [(-1000, 0)] + x_prof + [(1000, 0)]
    y_prof = [(-1000, 0)] + y_prof + [(1000, 0)]

    # X,Y DETECTOR POSITIONS FOR DEVICE
    x_vals = [-11.2 + 0.4*i for i in range(57)]
    y_vals = [-16.4 + 0.4*i for i in range(83)]

    # X,Y COORDINATES CORRECT, DO NOT INTERPOLATE
    if [i[0] for i in x_prof] == x_vals and [i[0] for i in y_prof] == y_vals:
        counts = ['Data:', '0', '0', '0', '0'] + \
            [str(int(1000*i[1])) for i in x_prof + y_prof] + \
            ['0', '0', '0', '0\n']


    else:  # INTERPOLATE X,Y COORDINATES ONTO DETECTOR POSITIONS
        interpolator = interp1d([i[0] for i in x_prof], [i[1] for i in x_prof])
        # counts_x = []
        counts_x = [float(i) for i in list(map(interpolator, x_vals))]

        interpolator = interp1d([i[0] for i in y_prof], [i[1] for i in y_prof])
        # counts_y = []
        counts_y = [float(i) for i in list(map(interpolator, y_vals))]

        # 1000 COUNTS PER CGY
        counts = ['Data:', '0', '0', '0', '0'] + \
            [str(int(1000*i)) for i in counts_x + counts_y] + \
            ['0', '0', '0', '0\n']



    prs_file = ['Version:\t {}\n'.format(7),
                'Filename:\t {} \n'.format('-'),
                'Date:\t {}\t Time:\t{}\n'.format(meas_date, meas_time),
                'Description:\t{}\n'.format(descrip),
                'Institution:\t{}\n'.format(instit),
                'Calibration File:\t{}\n'.format(file_cal),
                '\tProfiler Setup\n', '\n',
                'Collector Model:\t{}\n'.format(collector),
                'Collector Serial:\t{} Revision:\t{}\n'.format(
                    coll_sn, coll_rev),
                'Firmware Version:\t{}\n'.format(f_ware_ver),
                'Software Version:\t{}\n'.format(s_ware_v),
                'Buildup:\t{}\tcm\tWaterEquiv\n'.format(depth),
                'Nominal Gain\t{}\n'.format(nom_gain),
                'Orientation:\t{}\n'.format(orient),
                'SSD:\t{}\tcm\n'.format(ssd),
                'Beam Mode:\t{}\n'.format(bm_mode),
                'Tray Mount:\t{}\n'.format(tray),
                'Alignment:\t{}\n'.format(align),
                'Collection Interval:\t{}\n'.format(interval),
                '\n',
                '\tMachine Data\n',
                'Room:\t{}\n'.format(room),
                'Machine Type:\t{}\n'.format(mach_type),
                'Machine Model:\t{}\n'.format(mach_mod),
                'Machine Serial Number:\t{}\n'.format(mach_num),
                'Beam Type:\t{}\tEnergy:\t{} MeV\n'.format(
                    bm_modal, bm_energy),
                'Collimator:\tLeft: {} Right: {} Top: {} Bottom: {} cm\n'.format(
                   *jaws),
                'Wedge:\t{}\tat\t{}\n'.format(wdg_type, wdg_ang),
                'Rate:\t{}\tmu/Min\tDose:\t{}\n'.format(rate, dose),
                'Gantry Angle:\t{} deg\tCollimator Angle:\t{} deg\n'.format(
                    gantry, collimator),
                '\n',
                '\tData Flags\n',
                'Background Used:\t{}\n'.format(bkg_used),
                'Pulse Mode:\t{}\n'.format(pulse_mode),
                'Analyze Panels:\t{}\n'.format(alz_panels),
                '\n',
                '\tHardware Data\n',
                'Cal-Serial Number:\t{}\n'.format(cal_sn),
                'Cal-Revision:\t{}\n'.format(cal_rev),
                'Temperature:\t{}\n'.format(temp),
                'Dose Calibration\n',
                'Dose Per Count:\t{}\n'.format(dose_cal),
                'Dose:\t{}\n'.format(d_at_cal),
                'Absolute Calibration:\t{}\n'.format(abs_cal),
                'Energy:\t{} MV\n'.format(energy_cal),
                'TimeStamp:\t{} {}\n'.format(cal_date, cal_time),
                'Comments:\t{}\n'.format(cal_cmnts),
                'Gain Ratios for Amp0:\t\t{}\t{}\t{}\t{}\n'.format(1, 2, 4, 8),
                'Gain Ratios for Amp1:\t\t{}\t{}\t{}\t{}\n'.format(1, 2, 4, 8),
                'Gain Ratios for Amp2:\t\t{}\t{}\t{}\t{}\n'.format(1, 2, 4, 8),
                'Gain Ratios for Amp3:\t\t{}\t{}\t{}\t{}\n'.format(1, 2, 4, 8),
                '\n',
                'Multi Frame:\t{}\n'.format(m_frame),
                '# updates:\t{}\n'.format(updates),
                'Total Pulses:\t{}\n'.format(pulses),
                'Total Time:\t{}\n'.format(0.0),
                'Detectors:\t{}\t{}\t{}\t{}\n'.format(*detectors),
                'Detector Spacing:\t{}\n'.format(spacing),
                'Concatenation:\t{}\n'.format(concat),
                '\t'.join(['TYPE',
                           'UPDATE#',
                           'TIMETIC',
                           'PULSES',
                           'ERRORS'] +
                          ['X'+str(i) for i in range(1, 58)] +
                          ['Y'+str(i) for i in range(1, 84)] +
                          ['Z0', 'Z1', 'Z2', 'Z3\n']),
                '\t'.join(['BIAS1', '', '0', '',
                           ''] + ['0.0']*143 + ['0.0\n']),
                '\t'.join(['Calibration', '', '', '', ''] +
                          ['1.0']*139 + ['1.0\n']),
                '\t'.join(counts)]

    with open(file_name, 'w') as outfile:
        for line in prs_file:
            outfile.write(line)
    return prs_file



from glob import glob
from os.path import abspath, dirname, join as pjoin
from setuptools import setup, find_packages

root = dirname(abspath(__file__))


def execfile(fname, globs, locs=None):
    locs = locs or globs
    exec(compile(open(fname).read(), fname, "exec"), globs, locs)


source_path = 'src'
packages = find_packages(source_path)
root_packages = [
    package
    for package in packages
    if "." not in package
]

assert len(root_packages) == 1
package = root_packages[0]
package_directory = pjoin(root, source_path, package)


def get_variable_from_file(filepath, variable):
    filepath_in_package = pjoin(package_directory, filepath)
    globs = {}
    execfile(filepath_in_package, globs)
    variable_value = globs[variable]

    return variable_value


version = get_variable_from_file('_version.py', '__version__')
install_requires = get_variable_from_file(
    '_install_requires.py', 'install_requires')


setup(
    name=package,
    version=version,
    python_requires='>=3.7',
    author="PyMedPhys Contributors",
    author_email="developers@pymedphys.com",
    description='',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Scientific/Engineering :: Physics',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Healthcare Industry'
    ],
    packages=packages,
    package_dir={'': source_path},
    include_package_data=True,
    package_data={package: []},
    license='AGPL-3.0-or-later',
    install_requires=install_requires,
    extras_require={
        'test': [
            'pytest',
            'deepdiff'
        ]
    }
)



# Copyright (C) 2019 Pedro Martinez

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version (the "AGPL-3.0+").

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License and the additional terms for more
# details.

# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# ADDITIONAL TERMS are also included as allowed by Section 7 of the GNU
# Affero General Public License. These additional terms are Sections 1, 5,
# 6, 7, 8, and 9 from the Apache License, Version 2.0 (the "Apache-2.0")
# where all references to the definition "License" are instead defined to
# mean the AGPL-3.0+.

# You should have received a copy of the Apache-2.0 along with this
# program. If not, see <http://www.apache.org/licenses/LICENSE-2.0>.


###########################################################################################
#
#   Script name: qc-jaws
#
#   Description: Tool for calculating jaws junction shifts for linear accelerators.
#   The script opens every DICOM file in a given folder and creates a combined profile
#   resulting from the superposition of the two or more fields. It then detects the
#   peak/through formed by the gap/overlap of the fields. A window is then selected
#   around this point and a Savitzky-Golay smoothing filter is then applied to the
#   combined profile. This new curve is then used iteratively to minimize the
#   profile created every time one of the profiles slide to close the gap or decrease
#   the overlap. The profile will achieve it greatest level of homogeneity when the
#   dosimetric penumbra of both fields are matched in space. The final result is the
#   optimal calculation of the gap/overlap between the two profiles. The software
#   generates a pdf file with a summary of all the results.
#
#   Example usage: python qc-jaws "/folder/"
#
#   Author: Pedro Martinez
#   pedro.enrique.83@gmail.com
#   5877000722
#   Date:2019-04-09
#
###########################################################################################

import os
import sys
import pydicom
from PIL import *
import subprocess
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from tqdm import tqdm
import numpy as np
from scipy.signal import butter, filtfilt
from scipy.signal import savgol_filter
from scipy.stats import linregress
from scipy import signal
from scipy.signal import find_peaks, peak_prominences, peak_widths



def running_mean(x, N):
    out = np.zeros_like(x, dtype=np.float64)
    dim_len = x.shape[0]
    for i in range(dim_len):
        if N % 2 == 0:
            a, b = i - (N - 1) // 2, i + (N - 1) // 2 + 2
        else:
            a, b = i - (N - 1) // 2, i + (N - 1) // 2 + 1

        # cap indices to min and max indices
        a = max(0, a)
        b = min(dim_len, b)
        out[i] = np.mean(x[a:b])  # pylint: disable=unsupported-assignment-operation
    return out


# axial visualization and scrolling
def multi_slice_viewer(volume, dx, dy):
    # remove_keymap_conflicts({'j', 'k'})
    fig, ax = plt.subplots()
    ax.volume = volume
    ax.index = volume.shape[2] // 2
    print(ax.index)
    extent = (0, 0 + (volume.shape[1] * dx),
              0, 0 + (volume.shape[0] * dy))
    ax.imshow(volume[:, :, ax.index], extent=extent)
    ax.set_xlabel('x distance [mm]')
    ax.set_ylabel('y distance [mm]')
    ax.set_title("slice=" + str(ax.index))
    fig.suptitle('Axial view', fontsize=16)
    fig.canvas.mpl_connect('key_press_event', process_key_axial)


def process_key_axial(event):
    fig = event.canvas.figure
    ax = fig.axes[0]
    if event.key == 'j':
        previous_slice_axial(ax)
    elif event.key == 'k':
        next_slice_axial(ax)
    ax.set_title("slice=" + str(ax.index))
    fig.canvas.draw()


def previous_slice_axial(ax):
    volume = ax.volume
    ax.index = (ax.index - 1) % volume.shape[2]  # wrap around using %
    print(ax.index, volume.shape[2])
    ax.images[0].set_array(volume[:, :, ax.index])


def next_slice_axial(ax):
    volume = ax.volume
    ax.index = (ax.index + 1) % volume.shape[2]
    print(ax.index, volume.shape[2])
    ax.images[0].set_array(volume[:, :, ax.index])


def minimize_junction_Y(amplitude, peaks, peak_type, dx):
    amp_prev = 0
    amp_filt_prev = 0

    fig = plt.figure(figsize=(10, 6))  # create the plot

    kk = 1  # counter for figure generation
    for j in range(0, amplitude.shape[1] - 1):
        for k in range(j + 1, amplitude.shape[1]):  # looping through remaining images
            amp_base_res = signal.savgol_filter(amplitude[:, j], 1001, 3)
            amp_overlay_res = signal.savgol_filter(amplitude[:, k], 1001, 3)
            peak1, _ = find_peaks(amp_base_res, prominence=0.5)
            peak2, _ = find_peaks(amp_overlay_res, prominence=0.5)

            if abs(peak2 - peak1) < 2500:  # if the two peaks are close together proceeed to analysis
                cumsum_prev = 1e7
                if peak2<peak1:
                    amp_base_res = amplitude[:, k]
                    amp_overlay_res = amplitude[:, j]
                else:
                    amp_base_res = amplitude[:, j]
                    amp_overlay_res = amplitude[:, k]

                if peak_type[kk-1] == 0: #kk-1 to start from the null element of the array
                    inc = -1
                else:
                    inc = 1
                for i in range(0, inc * 80, inc * 1):
                    # print('i', i)
                    x = np.linspace(0, 0 + (len(amp_base_res) * dx), len(amplitude),
                                    endpoint=False)  # definition of the distance axis
                    amp_overlay_res_roll = np.roll(amp_overlay_res, i)

                    # amplitude is the vector to analyze +-500 samples from the center
                    amp_tot = (amp_base_res[peaks[kk-1] - 1000:peaks[kk-1] + 1000] + amp_overlay_res_roll[peaks[kk-1] - 1000:peaks[kk-1] + 1000])  # divided by 2 to normalize

                    xsel = x[peaks[kk-1] - 1000:peaks[kk-1] + 1000]

                    amp_filt = running_mean(amp_tot, 281)
                    cumsum = np.sum(np.abs(amp_tot - amp_filt))

                    if cumsum > cumsum_prev:  # then we went too far
                        ax = fig.add_subplot(amplitude.shape[1] - 1, 1, kk)
                        ax.plot(amp_prev)
                        ax.plot(amp_filt_prev)
                        if kk == 1:
                            ax.set_title('Minimization result',fontsize=16)
                        if kk == amplitude.shape[1] - 1:  # if we reach the final plot the add the x axis label
                            ax.set_xlabel('distance [mm]')

                        ax.set_ylabel('amplitude')
                        ax.annotate('delta=' + str(abs(i - inc * 1) * dx) + ' mm', xy=(2, 1), xycoords='axes fraction',
                                    xytext=(.35, .10))

                        kk = kk + 1
                        break
                    else:
                        amp_prev = amp_tot
                        amp_filt_prev = amp_filt
                        cumsum_prev = cumsum



            else:
                print(j, k, 'the data is not contiguous finding another curve in dataset')

    return fig










def minimize_junction_X(amplitude, peaks, peak_type, dx):
    amp_prev = 0
    amp_filt_prev = 0

    fig = plt.figure(figsize=(10, 6))  # create the plot

    kk = 1  # counter for figure generation
    for j in range(0, amplitude.shape[1] - 1):
        for k in range(j + 1, amplitude.shape[1]):  # looping through remaining images
            amp_base_res = signal.savgol_filter(amplitude[:, j], 1001, 3)
            amp_overlay_res = signal.savgol_filter(amplitude[:, k], 1001, 3)
            peak1, _ = find_peaks(amp_base_res, prominence=0.5)
            peak2, _ = find_peaks(amp_overlay_res, prominence=0.5)

            if abs(peak2 - peak1) < 2500:  # if the two peaks are close together proceeed to analysis
                cumsum_prev = 1e7
                if peak2 < peak1: # this guarantee that we always slide the overlay
                    amp_base_res = amplitude[:, k]
                    amp_overlay_res = amplitude[:, j]
                else:
                    amp_base_res = amplitude[:, j]
                    amp_overlay_res = amplitude[:, k]

                if peak_type[j] == 0:
                    inc = -1
                else:
                    inc = 1
                for i in range(0, inc * 80, inc * 1):
                    x = np.linspace(0, 0 + (len(amp_base_res) * dx), len(amplitude),
                                    endpoint=False)  # definition of the distance axis
                    amp_overlay_res_roll = np.roll(amp_overlay_res, i)

                    # amplitude is the vector to analyze +-500 samples from the center
                    amp_tot = (amp_base_res[peaks[j] - 1000:peaks[j] + 1000] + amp_overlay_res_roll[
                                                                               peaks[j] - 1000:peaks[
                                                                                                   j] + 1000])  # divided by 2 to normalize
                    xsel = x[peaks[j] - 1000:peaks[j] + 1000]

                    amp_filt = running_mean(amp_tot, 281)
                    cumsum = np.sum(np.abs(amp_tot - amp_filt))

                    if cumsum > cumsum_prev:  # then we went too far
                        ax = fig.add_subplot(amplitude.shape[1] - 1, 1, kk)
                        ax.plot(amp_prev)
                        ax.plot(amp_filt_prev)
                        if kk == 1:
                            ax.set_title('Minimization result',fontsize=16)
                        if kk == amplitude.shape[1] - 1:  # if we reach the final plot the add the x axis label
                            ax.set_xlabel('distance [mm]')

                        ax.set_ylabel('amplitude')
                        ax.annotate('delta=' + str(abs(i - inc * 1) * dx) + ' mm', xy=(2, 1), xycoords='axes fraction',
                                    xytext=(.35, .10))


                        kk = kk + 1
                        break
                    else:
                        amp_prev = amp_tot
                        amp_filt_prev = amp_filt
                        cumsum_prev = cumsum



            else:
                print(j, k, 'the data is not contiguous finding another curve in dataset')

    return fig













#minimize junction for field rotations is done differently given the shape of the fields
def minimize_junction_fieldrot(amplitude, peaks, peak_type, dx, profilename):

    amp_prev = 0
    amp_filt_prev = 0

    fig = plt.figure(figsize=(10, 6))  # create the plot

    kk = 1  # counter for figure generation
    for j in range(0, amplitude.shape[1] - 1):
        for k in range(j + 1, amplitude.shape[1]):  # looping through remaining images
            amp_base_res = signal.savgol_filter(amplitude[:, j], 1001, 3)
            amp_overlay_res = signal.savgol_filter(amplitude[:, k], 1001, 3)
            peak1, _ = find_peaks(amp_base_res, prominence=0.5)
            peak2, _ = find_peaks(amp_overlay_res, prominence=0.5)

            cumsum_prev = 1e7
            amp_base_res = amplitude[:, j]
            amp_overlay_res = amplitude[:, k]

            if peak_type[j] == 0:
                inc = -1
            else:
                inc = 1
            for i in range(0, inc * 80, inc * 1):
                x = np.linspace(0, 0 + (len(amp_base_res) * dx), len(amplitude),
                                endpoint=False)  # definition of the distance axis
                amp_overlay_res_roll = np.roll(amp_overlay_res, i)

                amp_tot = (amp_base_res[peaks[j] - 1000:peaks[j] + 1000] + amp_overlay_res_roll[peaks[j] - 1000:peaks[
                                                                                                                   j] + 1000])  # divided by 2 to normalize
                xsel = x[peaks[j] - 1000:peaks[j] + 1000]
                amp_filt = running_mean(amp_tot, 281)

                cumsum = np.sum(np.abs(amp_tot - amp_filt))

                if cumsum > cumsum_prev:  # then we went too far
                    ax = fig.add_subplot(amplitude.shape[1] - 1, 1, kk)

                    ax.plot(amp_prev)
                    ax.plot(amp_filt_prev)
                    if kk == 1:
                        ax.set_title('Minimization result - '+profilename,fontsize=16)
                    if kk == amplitude.shape[1] - 1:  # if we reach the final plot the add the x axis label
                        ax.set_xlabel('distance [mm]')

                    ax.set_ylabel('amplitude')
                    ax.annotate('delta=' + str(abs(i - inc * 1) * dx) + ' mm', xy=(2, 1), xycoords='axes fraction',
                                xytext=(.35, .10))


                    kk = kk + 1
                    break
                else:
                    amp_prev = amp_tot
                    amp_filt_prev = amp_filt
                    cumsum_prev = cumsum


    return fig











# this subroutine aims to find the peaks
def peak_find(ampl_resamp, dx):
    peak_figs = []
    peaks = []
    peak_type = []
    for j in range(0, ampl_resamp.shape[1] - 1):
        amp_base_res = signal.savgol_filter(ampl_resamp[:, j], 1501, 1)
        for k in range(j + 1, ampl_resamp.shape[1]):
            amp_overlay_res = signal.savgol_filter(ampl_resamp[:, k], 1501, 1)

            peak1, _ = find_peaks(amp_base_res, prominence=0.5)
            peak2, _ = find_peaks(amp_overlay_res, prominence=0.5)


            if abs(peak2 - peak1) < 2500:  # if the two peaks are separated the two fields are not adjacent.
                amp_peak = (ampl_resamp[:, j] + ampl_resamp[:, k])
                x = np.linspace(0, 0 + (len(amp_peak) * dx / 10), len(amp_peak),
                                endpoint=False)  # definition of the distance axis


                peak_pos, _ = find_peaks(signal.savgol_filter(amp_peak[min(peak1[0],peak2[0]):max(peak1[0],peak2[0])],201,3), prominence=0.010)
                peak_neg, _ = find_peaks(signal.savgol_filter(-amp_peak[min(peak1[0], peak2[0]):max(peak1[0], peak2[0])], 201, 3), prominence=0.010)

                if len(peak_pos)==1 and len(peak_neg) != 1:
                    peak = peak_pos
                    peaks.append(min(peak1[0],peak2[0])+peak[0])
                    peak_type.append(1)

                    fig = plt.figure(figsize=(10, 6))
                    plt.plot(x, amp_peak, label='Total amplitude profile')
                    plt.plot(x[min(peak1[0], peak2[0]) + peak[0]], amp_peak[min(peak1[0], peak2[0]) + peak[0]], "x",
                             label='Peaks detected')
                    plt.ylabel('amplitude [a.u.]')
                    plt.xlabel('distance [mm]')
                    plt.legend()
                    fig.suptitle('Junctions', fontsize=16)
                    peak_figs.append(fig)

                elif len(peak_pos) != 1 and len(peak_neg) == 1:
                    peak = peak_neg
                    peak_type.append(0)
                    peaks.append(min(peak1[0],peak2[0])+peak[0])

                    fig = plt.figure(figsize=(10, 6))
                    plt.plot(x, amp_peak, label='Total amplitude profile')
                    plt.plot(x[min(peak1[0], peak2[0]) + peak[0]], amp_peak[min(peak1[0], peak2[0]) + peak[0]], "x",
                             label='Peaks detected')
                    plt.ylabel('amplitude [a.u.]')
                    plt.xlabel('distance [mm]')
                    plt.legend()
                    fig.suptitle('Junctions', fontsize=16)
                    peak_figs.append(fig)

                else:
                    peaks.append(0)
                    peak_type.append(0)
                    fig = plt.figure(figsize=(10, 6))
                    plt.plot(x, amp_peak, label='Total amplitude profile')
                    plt.plot(x[min(peak1[0], peak2[0]) ], amp_peak[min(peak1[0], peak2[0]) ], "x",
                             label='Peaks detected')
                    plt.ylabel('amplitude [a.u.]')
                    plt.xlabel('distance [mm]')
                    plt.legend()
                    fig.suptitle('Junctions', fontsize=16)
                    peak_figs.append(fig)





            else:
                print(j, k, 'the data is not contiguous finding another curve in dataset')

    return peaks, peak_type, peak_figs













# this subroutine aims to find the peaks
def peak_find_fieldrot(ampl_resamp, dx,profilename):
    peaks = []
    peak_type = []
    for j in range(0, ampl_resamp.shape[1] - 1):
        amp_base_res = signal.savgol_filter(ampl_resamp[:, j], 1501, 1)
        for k in range(j + 1, ampl_resamp.shape[1]):
            amp_overlay_res = signal.savgol_filter(ampl_resamp[:, k], 1501, 1)


            peak1, _ = find_peaks(amp_base_res, prominence=0.5)
            peak2, _ = find_peaks(amp_overlay_res, prominence=0.5)
            # print('peak find', peak1, peak2, abs(peak2 - peak1))

            if abs(peak2 - peak1) <= 4000:  # if the two peaks are separated the two fields are not adjacent.
                amp_peak = (ampl_resamp[:, j] + ampl_resamp[:, k]) / 2
                x = np.linspace(0, 0 + (len(amp_peak) * dx / 10), len(amp_peak),
                                endpoint=False)  # definition of the distance axis

                peak_pos, _ = find_peaks(signal.savgol_filter(amp_peak[min(peak1[0],peak2[0]):max(peak1[0],peak2[0])],201,3), prominence=0.010)
                peak_neg, _ = find_peaks(signal.savgol_filter(-amp_peak[min(peak1[0], peak2[0]):max(peak1[0], peak2[0])], 201, 3), prominence=0.010)

                if len(peak_pos)==1 and len(peak_neg) != 1:
                    peak = peak_pos
                    # peak, _ = find_peaks(-amp_peak, prominence=1000)
                    peaks.append(min(peak1[0],peak2[0])+peak[0])
                    peak_type.append(1)

                    fig = plt.figure(figsize=(10, 6))
                    plt.plot(x, amp_peak, label='Total amplitude profile - ' + profilename)
                    plt.plot(x[min(peak1[0], peak2[0]) + peak[0]], amp_peak[min(peak1[0], peak2[0]) + peak[0]], "x",
                             label='Peaks detected')
                    plt.ylabel('amplitude [a.u.]')
                    plt.xlabel('distance [mm]')
                    plt.legend()
                    fig.suptitle('Junctions - ' + profilename, fontsize=16)



                elif len(peak_pos) != 1 and len(peak_neg) == 1:
                    peak = peak_neg
                    peak_type.append(0)
                    peaks.append(min(peak1[0],peak2[0])+peak[0])

                    fig = plt.figure(figsize=(10, 6))
                    plt.plot(x, amp_peak, label='Total amplitude profile - ' + profilename)
                    plt.plot(x[min(peak1[0], peak2[0]) + peak[0]], amp_peak[min(peak1[0], peak2[0]) + peak[0]], "x",
                             label='Peaks detected')
                    plt.ylabel('amplitude [a.u.]')
                    plt.xlabel('distance [mm]')
                    plt.legend()
                    fig.suptitle('Junctions - ' + profilename, fontsize=16)

                else:
                    peaks.append(0)
                    peak_type.append(0)
                    fig = plt.figure(figsize=(10, 6))
                    plt.plot(x, amp_peak, label='Total amplitude profile - ' + profilename)
                    plt.plot(x[min(peak1[0], peak2[0]) ], amp_peak[min(peak1[0], peak2[0]) ], "x",
                             label='Peaks detected')
                    plt.ylabel('amplitude [a.u.]')
                    plt.xlabel('distance [mm]')
                    plt.legend()
                    fig.suptitle('Junctions - ' + profilename, fontsize=16)



            else:
                print(j, k, 'the data is not contiguous finding another curve in dataset')

    return peaks, peak_type, fig














# this subroutine will merge the two jaws into a single image and display a graph of the overlap
def merge_view_vert(volume, dx, dy):
    junctions = []

    # creating merged volume
    merge_vol = np.zeros((volume.shape[0], volume.shape[1]))

    # creating vector for processing along cols (one row)
    amplitude = np.zeros((volume.shape[1], volume.shape[2]))  # 1 if it is vertical 0 if the bars are horizontal

    x = np.linspace(0, 0 + (volume.shape[1] * dx), volume.shape[1], endpoint=False)  # definition of the distance axis

    # merging the two images together
    ampl_resamp = np.zeros(((volume.shape[1]) * 10, volume.shape[2]))

    for slice in tqdm(range(0, volume.shape[2])):
        merge_vol = merge_vol + volume[:, :, slice]
        amplitude[:, slice] = volume[int(volume.shape[0] / 2), :, slice]
        ampl_resamp[:, slice] = signal.resample(amplitude[:, slice],
                                                int(len(amplitude)) * 10)  # resampling the amplitude vector

    fig, ax = plt.subplots(nrows=2, squeeze=True, figsize=(6, 8))

    extent = (0, 0 + (volume.shape[1] * dx),
              0, 0 + (volume.shape[0] * dy))

    ax[0].imshow(merge_vol, extent=extent)
    ax[0].set_xlabel('x distance [mm]')
    ax[0].set_ylabel('y distance [mm]')

    ax[1].plot(x, amplitude)
    ax[1].set_xlabel('x distance [mm]')
    ax[1].set_ylabel('amplitude')
    ax[1].legend()
    fig.suptitle('Merged volume', fontsize=16)

    peaks, peak_type, peak_figs = peak_find(ampl_resamp, dx)
    junction_figs = minimize_junction_X(ampl_resamp, peaks, peak_type, dx / 10)
    junctions.append(junction_figs)

    return fig, peak_figs, junctions










# this subroutine will merge the 4 jaws and analyze them
# each jaw is 6cm in length (60mm)
def merge_view_horz(volume, dx, dy):
    junctions = []

    # creating merged volume
    merge_vol = np.zeros((volume.shape[0], volume.shape[1]))

    # creating vector for processing along cols (one row)
    amplitude = np.zeros((volume.shape[0], volume.shape[2]))  # 1 if it is vertical 0 if the bars are horizontal

    y = np.linspace(0, 0 + (volume.shape[0] * dy), volume.shape[0], endpoint=False)  # definition of the distance axis

    # merging the two images together
    ampl_resamp = np.zeros(((volume.shape[0]) * 10, volume.shape[2]))

    for slice in tqdm(range(0, volume.shape[2])):
        merge_vol = merge_vol + volume[:, :, slice]
        amplitude[:, slice] = volume[:, int(volume.shape[1] / 2), slice]
        ampl_resamp[:, slice] = signal.resample(amplitude[:, slice],
                                                int(len(amplitude)) * 10)  # resampling the amplitude vector

    fig, ax = plt.subplots(nrows=2, squeeze=True, figsize=(6, 8))

    extent = (0, 0 + (volume.shape[1] * dx),
              0, 0 + (volume.shape[0] * dy))



    ax[0].imshow(merge_vol, extent=extent, aspect='auto')
    ax[0].set_xlabel('x distance [mm]')
    ax[0].set_ylabel('y distance [mm]')

    ax[1].plot(y, amplitude, label='Amplitude profile')
    ax[1].set_ylabel('amplitude')
    ax[1].set_xlabel('y distance [mm]')
    ax[1].legend()
    fig.suptitle('Merged volume', fontsize=16)

    peaks, peak_type, peak_figs = peak_find(ampl_resamp, dy)
    junction_figs = minimize_junction_Y(ampl_resamp, peaks, peak_type, dy / 10)
    junctions.append(junction_figs)

    return fig, peak_figs, junctions















# this subroutine will merge the 4 jaws and analyze the two upper and two lower pairs
# each jaw is 6cm in length (60mm)
def merge_view_filtrot(volume, dx, dy):

    volume_resort = np.copy(volume)  # this will hold the resorted volume 0 to 3 clockwise
    junctions_comb = []
    peaks_figs_comb = []





    # we need to create 4 matches

    # 0,1,2,3 will be tagged top left, top right, bottom right, bottom left
    for i in range(0, int(np.shape(volume)[2])):
        diag_stack = [0, 0, 0, 0]  # we will sum along one direction whichever is biggest will tag the file
        for j in range(0, int(min([np.shape(volume)[0], np.shape(volume)[1]]) / 2)):
            # print('j=',j,int(np.shape(volume)[0] / 2)+j, int(np.shape(volume)[1] / 2)+j)
            diag_stack[0] = diag_stack[0] + volume[
                int(np.shape(volume)[0] / 2) - j, int(np.shape(volume)[1] / 2) - j, i]
            diag_stack[1] = diag_stack[1] + volume[
                int(np.shape(volume)[0] / 2) - j, int(np.shape(volume)[1] / 2) + j, i]
            diag_stack[2] = diag_stack[2] + volume[
                int(np.shape(volume)[0] / 2) + j, int(np.shape(volume)[1] / 2) + j, i]
            diag_stack[3] = diag_stack[3] + volume[
                int(np.shape(volume)[0] / 2) + j, int(np.shape(volume)[1] / 2) - j, i]

        volume_resort[:, :, np.argmax(diag_stack)] = volume[:, :, i]


    # creating merged volumes
    merge_vol = np.zeros((volume_resort.shape[0], volume_resort.shape[1]))

    # creating vector for processing (1 horizontal & 1 vertical)
    amplitude_horz = np.zeros(
        (volume_resort.shape[1], volume_resort.shape[2]))  # 1 if it is vertical 0 if the bars are horizontal
    amplitude_vert = np.zeros((volume_resort.shape[0], volume_resort.shape[2]))

    y = np.linspace(0, 0 + (volume_resort.shape[0] * dy), volume_resort.shape[0],
                    endpoint=False)  # definition of the distance axis
    x = np.linspace(0, 0 + (volume_resort.shape[1] * dy), volume_resort.shape[1],
                    endpoint=False)  # definition of the distance axis

    ampl_resamp_y1 = np.zeros(((volume_resort.shape[0]) * 10, int(volume_resort.shape[2] / 2)))
    ampl_resamp_y2 = np.zeros(((volume_resort.shape[0]) * 10, int(volume_resort.shape[2] / 2)))

    ampl_resamp_x1 = np.zeros(((volume_resort.shape[1]) * 10, int(volume_resort.shape[2] / 2)))
    ampl_resamp_x2 = np.zeros(((volume_resort.shape[1]) * 10, int(volume_resort.shape[2] / 2)))


    amplitude_horz[:, 0] = volume_resort[int(volume_resort.shape[0] / 3.25), :, 0]    # for profile 1
    amplitude_horz[:, 1] = volume_resort[int(volume_resort.shape[0] / 3.25), :, 1]    # for profile 1
    amplitude_horz[:, 3] = volume_resort[int(volume_resort.shape[0]) - int(volume_resort.shape[0] / 3.25), :,
                           2]  # the numbers here are reversed because we are going to slide the second graph (the overlay) to minimize the error  #for profile 2
    amplitude_horz[:, 2] = volume_resort[int(volume_resort.shape[0]) - int(volume_resort.shape[0] / 3.25), :, 3]

    amplitude_vert[:, 0] = volume_resort[:, int(volume_resort.shape[1]) - int(volume_resort.shape[1] / 2.8),
                           1]  # the numbers here are reversed because we are going to slide the second graph (the overlay) to minimize the error #for profile 3
    amplitude_vert[:, 1] = volume_resort[:, int(volume_resort.shape[1]) - int(volume_resort.shape[1] / 2.8), 2]
    amplitude_vert[:, 3] = volume_resort[:, int(volume_resort.shape[1] / 2.8), 3]   #for profile 4
    amplitude_vert[:, 2] = volume_resort[:, int(volume_resort.shape[1] / 2.8), 0]

    plt.figure()
    for slice in tqdm(range(0, int(volume.shape[2] / 2))):
        merge_vol = merge_vol + volume[:, :, slice]

        data_samp = amplitude_vert[:, slice]
        ampl_resamp_y1[:, slice] = signal.resample(data_samp, int(np.shape(amplitude_vert)[0]) * 10)
        data_samp = amplitude_horz[:, slice]
        ampl_resamp_x1[:, slice] = signal.resample(data_samp, int(np.shape(amplitude_horz)[0]) * 10)




    for slice in tqdm(range(int(volume.shape[2] / 2), volume.shape[2])):
        merge_vol = merge_vol + volume[:, :, slice]
        data_samp = amplitude_vert[:, slice]
        ampl_resamp_y2[:, slice - int(volume.shape[2] / 2)] = signal.resample(data_samp,
                                                                              int(np.shape(amplitude_vert)[0]) * 10)
        data_samp = amplitude_horz[:, slice]
        ampl_resamp_x2[:, slice - int(volume.shape[2] / 2)] = signal.resample(data_samp,
                                                                              int(np.shape(amplitude_horz)[0]) * 10)


    fig, ax = plt.subplots(ncols=1, nrows=1, squeeze=True, figsize=(6, 8))


    extent = (0, 0 + (volume.shape[1] * dx),
              0, 0 + (volume.shape[0] * dy))

    ax.imshow(merge_vol, extent=extent, aspect='auto')
    ax.set_aspect('equal', 'box')
    ax.set_xlabel('x distance [mm]')
    ax.set_ylabel('y distance [mm]')






    ax.hlines(dy * int(volume_resort.shape[0] / 3.25), 0, dx * volume_resort.shape[1])
    ax.text(dx * int(volume_resort.shape[1]/2.25), dy * int(volume_resort.shape[0] / 3),'Profile 2')

    ax.hlines(dy * int(volume_resort.shape[0]) - dy*int(volume_resort.shape[0] / 3.25), 0, dx * volume_resort.shape[1])
    ax.text(dx * int(volume_resort.shape[1] / 2.25), dy * int(volume_resort.shape[0]) - dy*int(volume_resort.shape[0] / 3.5), 'Profile 1')

    ax.vlines(dx * int(volume_resort.shape[1] / 2.8), 0, dy * volume_resort.shape[0])
    ax.text(dx * int(volume_resort.shape[1] / 3.1), dy * int(volume_resort.shape[0]/1.8), 'Profile 4',rotation=90)

    ax.vlines(dx * int(volume_resort.shape[1]) - dx*int(volume_resort.shape[1] / 2.8), 0, dy * volume_resort.shape[0])
    ax.text(dx * int(volume_resort.shape[1]) - dx*int(volume_resort.shape[1] / 2.9), dy * int(volume_resort.shape[0]/1.8), 'Profile 3',rotation=90)

    peaks, peak_type, peak_figs = peak_find_fieldrot(ampl_resamp_x1, dx,'Profile 1')
    junction_figs = minimize_junction_fieldrot(ampl_resamp_x1, peaks, peak_type, dx / 10,'Profile 1')
    peaks_figs_comb.append(peak_figs)
    junctions_comb.append(junction_figs)

    peaks, peak_type, peak_figs = peak_find_fieldrot(ampl_resamp_x2, dx,'Profile 2')
    junction_figs = minimize_junction_fieldrot(ampl_resamp_x2, peaks, peak_type, dx / 10, 'Profile 2')
    peaks_figs_comb.append(peak_figs)
    junctions_comb.append(junction_figs)

    peaks, peak_type, peak_figs = peak_find_fieldrot(ampl_resamp_y1, dy,'Profile 3')
    junction_figs = minimize_junction_fieldrot(ampl_resamp_y1, peaks, peak_type, dy / 10 , 'Profile 3')
    peaks_figs_comb.append(peak_figs)
    junctions_comb.append(junction_figs)

    peaks, peak_type, peak_figs = peak_find_fieldrot(ampl_resamp_y2, dy,'Profile 4')
    junction_figs = minimize_junction_fieldrot(ampl_resamp_y2, peaks, peak_type, dy / 10, 'Profile 4')
    peaks_figs_comb.append(peak_figs)
    junctions_comb.append(junction_figs)

    return fig, peaks_figs_comb, junctions_comb



# this routine anlyzes the volume and autodetect the images and categorizes them for different tests
def image_analyze(volume,ioption):
    xfield = []
    yfield = []
    rotfield = []

    if ioption.startswith(('y', 'yeah', 'yes')):
        max_val=np.amax(volume)
        volume = volume/max_val
        min_val = np.amin(volume)
        volume = volume - min_val
        volume = (1 - volume) # inverting the range

        min_val = np.amin(volume) #normalizing
        volume=volume-min_val
        volume=volume/(np.amax(volume))



        # print('Volume shape=', np.shape(volume), np.amin([np.shape(volume)[0], np.shape(volume)[1]]))
        kx = 0
        ky = 0
        krot = 0
        for slice in range(0, volume.shape[2]):
            stack1 = np.sum(volume[
                            int(np.shape(volume)[0] / 2 - np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2):int(
                                np.shape(volume)[0] / 2 + np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2),
                            int(np.shape(volume)[1] / 2 - np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2):int(
                                np.shape(volume)[1] / 2 + np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2),
                            slice], axis=0)
            maxstack1 = np.amax(stack1)

            stack2 = np.sum(volume[
                            int(np.shape(volume)[0] / 2 - np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2):int(
                                np.shape(volume)[0] / 2 + np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2),
                            int(np.shape(volume)[1] / 2 - np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2):int(
                                np.shape(volume)[1] / 2 + np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2),
                            slice],
                            axis=1)
            maxstack2 = np.amax(stack2)


            if maxstack2 / maxstack1 > 1.1:  # It is a Y field folder
                if ky == 0:
                    yfield = volume[:, :, slice]
                    yfield = yfield[:, :, np.newaxis]
                else:
                    volappend = volume[:, :, slice]
                    yfield = np.append(yfield, volappend[:, :, np.newaxis], axis=2)
                ky = ky + 1
            elif maxstack2 / maxstack1 < 0.9:  # It is a X field folder
                if kx == 0:
                    xfield = volume[:, :, slice]
                    xfield = xfield[:, :, np.newaxis]
                else:
                    volappend = volume[:, :, slice]
                    xfield = np.append(xfield, volappend[:, :, np.newaxis], axis=2)
                kx = kx + 1
            else:  # It is a field rotation folder
                if krot == 0:
                    rotfield = volume[:, :, slice]
                    rotfield = rotfield[:, :, np.newaxis]
                else:
                    volappend = volume[:, :, slice]
                    rotfield = np.append(rotfield, volappend[:, :, np.newaxis], axis=2)
                krot = krot + 1





    else:
        min_val = np.amin(volume)
        volume=volume-min_val
        volume=volume/(np.amax(volume))
        kx=0
        ky = 0
        krot=0
        for slice in range(0, volume.shape[2]):
            stack1 = np.sum(volume[int(np.shape(volume)[0]/2-np.amin([np.shape(volume)[0],np.shape(volume)[1]])/2):int(np.shape(volume)[0]/2+np.amin([np.shape(volume)[0],np.shape(volume)[1]])/2),int(np.shape(volume)[1]/2-np.amin([np.shape(volume)[0],np.shape(volume)[1]])/2) :int(np.shape(volume)[1]/2+np.amin([np.shape(volume)[0],np.shape(volume)[1]])/2), slice], axis=0)
            maxstack1 = np.amax(stack1)

            stack2 = np.sum(volume[
                            int(np.shape(volume)[0] / 2 - np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2):int(
                                np.shape(volume)[0] / 2 + np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2),
                            int(np.shape(volume)[1] / 2 - np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2):int(
                                np.shape(volume)[1] / 2 + np.amin([np.shape(volume)[0], np.shape(volume)[1]]) / 2), slice],
                            axis=1)
            maxstack2 = np.amax(stack2)



            if maxstack2 / maxstack1 > 1.5:  # It is a Y field folder
                if ky==0:
                    yfield = volume[:, :, slice]
                    yfield = yfield[:, :, np.newaxis]
                else:
                    volappend = volume[:, :, slice]
                    yfield = np.append(yfield, volappend[:,:,np.newaxis],axis=2)
                ky=ky+1
            elif maxstack2 / maxstack1 < 0.5:  # It is a X field folder
                if kx==0:
                    xfield = volume[:, :, slice]
                    xfield = xfield[:, :, np.newaxis]
                else:
                    volappend = volume[:, :, slice]
                    xfield = np.append(xfield, volappend[:,:,np.newaxis],axis=2)
                kx = kx + 1
            else:    # It is a field rotation folder
                if krot==0:
                    rotfield = volume[:, :, slice]
                    rotfield=rotfield[:,:,np.newaxis]
                else:
                    volappend = volume[:, :, slice]
                    rotfield = np.append(rotfield, volappend[:,:,np.newaxis],axis=2)
                krot = krot + 1



    return xfield, yfield, rotfield





# this routine anlyzes the volume and autodetect what type of analysis to carry on (x, Y, Field Rot)
def folder_analyze(volume):
    for slice in range(0, volume.shape[2]):
        stack1 = np.sum(volume[:, :, slice], axis=0)
        maxstack1 = np.max(stack1)

        stack2 = np.sum(volume[:, :, slice], axis=1)
        maxstack2 = np.max(stack2)

        if maxstack2 / maxstack1 > 1.5:  # It is a Y field folder
            return 2
        elif maxstack2 / maxstack1 < 0.5:  # It is a X field folder
            return 1
        else:
            return 3  # It is a field rotation folder



def read_dicom3D(dirname, poption,ioption):
    slice = 0
    # lstFilesDCM = [] #empty list to store dicom files
    for subdir, dirs, files in os.walk(dirname):
        k = 0
        for file in tqdm(sorted(files)):
            print('filename=', file)
            if os.path.splitext(file)[1]=='.dcm':
                if poption.startswith(('y', 'yeah', 'yes')):
                    subprocess.call(
                        ["gdcmconv", "-w", dirname + file, os.path.splitext(dirname + file)[0] + "_decomp" + ".dcm"])
                    dataset = pydicom.dcmread(os.path.splitext(dirname + file)[0] + "_decomp" + ".dcm")
                    if k == 0:
                        ArrayDicom = np.zeros((dataset.Rows, dataset.Columns), dtype=dataset.pixel_array.dtype)
                        ArrayDicom = np.dstack((ArrayDicom, dataset.pixel_array))
                        # print("slice thickness [mm]=",dataset.SliceThickness)
                        SID = dataset.RTImageSID
                        dx = 1 / (SID * (1 / dataset.ImagePlanePixelSpacing[0]) / 1000)
                        dy = 1 / (SID * (1 / dataset.ImagePlanePixelSpacing[1]) / 1000)
                        print("pixel spacing row [mm]=", dx)
                        print("pixel spacing col [mm]=", dy)
                    else:
                        ArrayDicom = np.dstack((ArrayDicom, dataset.pixel_array))
                elif poption.startswith(('n', 'no', 'nope')):
                    dataset = pydicom.dcmread(dirname + file)
                    if k == 0:
                        ArrayDicom = np.zeros((dataset.Rows, dataset.Columns, 0), dtype=dataset.pixel_array.dtype)
                        ArrayDicom = np.dstack((ArrayDicom, dataset.pixel_array))
                        # print("slice thickness [mm]=", dataset.SliceThickness)
                        SID = dataset.RTImageSID
                        dx = 1 / (SID * (1 / dataset.ImagePlanePixelSpacing[0]) / 1000)
                        dy = 1 / (SID * (1 / dataset.ImagePlanePixelSpacing[1]) / 1000)
                        print("pixel spacing row [mm]=", dx)
                        print("pixel spacing col [mm]=", dy)
                    else:
                        ArrayDicom = np.dstack((ArrayDicom, dataset.pixel_array))
                print(k)
                k = k + 1



    xfield,yfield,rotfield = image_analyze(ArrayDicom,ioption)


    multi_slice_viewer(ArrayDicom, dx, dy)


    fig, peak_figs, junctions_figs = merge_view_vert(xfield, dx, dy)
    with PdfPages(dirname + 'jaws_X_report.pdf') as pdf:
        pdf.savefig(fig)
        for i in range(0, len(peak_figs)):
            pdf.savefig(peak_figs[i])

        for i in range(0, len(junctions_figs)):
            pdf.savefig(junctions_figs[i])

        plt.close()



    fig, peak_figs, junctions_figs = merge_view_horz(yfield, dx, dy)
    with PdfPages(dirname + 'jaws_Y_report.pdf') as pdf:
        pdf.savefig(fig)
        for i in range(0, len(peak_figs)):
            pdf.savefig(peak_figs[i])

        for i in range(0, len(junctions_figs)):
            pdf.savefig(junctions_figs[i])

        plt.close()




    fig, peak_figs, junctions_figs = merge_view_filtrot(rotfield, dx, dy)
    with PdfPages(dirname + 'jaws_FR_report.pdf') as pdf:
        pdf.savefig(fig)
        for i in range(0, len(peak_figs)):
            pdf.savefig(peak_figs[i])

        for i in range(0, len(junctions_figs)):
            pdf.savefig(junctions_figs[i])

        plt.close()








    # Normal mode:
    print()
    print("Directory folder.........:", dirname)
    print("Storage type.....:", dataset.SOPClassUID)
    print()

    pat_name = dataset.PatientName
    display_name = pat_name.family_name + ", " + pat_name.given_name
    print("Patient's name...:", display_name)
    print("Patient id.......:", dataset.PatientID)
    print("Modality.........:", dataset.Modality)
    print("Study Date.......:", dataset.StudyDate)
    print("Gantry angle......", dataset.GantryAngle)






def main():
    try:
        dirname = str(sys.argv[1])
        print(dirname)
    except:
        print('Please enter a valid filename')
        print("Use the following command to run this script")
        print("python test_pydicom3D.py \"[dirname]\"")
    
    while True:  # example of infinite loops using try and except to catch only numbers
        line = input('Are the files compressed [yes(y)/no(n)]> ')
        try:
            ##        if line == 'done':
            ##            break
            poption = str(line.lower())
            if poption.startswith(('y', 'yeah', 'yes', 'n', 'no', 'nope')):
                break
    
        except:
            print('Please enter a valid option:')
    
    
    while True:  # example of infinite loops using try and except to catch only numbers
        line = input('Are these files from a clinac [yes(y)/no(n)]> ')
        try:
            ##        if line == 'done':
            ##            break
            ioption = str(line.lower())
            if ioption.startswith(('y', 'yeah', 'yes', 'n', 'no', 'nope')):
                break
    
        except:
            print('Please enter a valid option:')
    
    
    
    read_dicom3D(dirname, poption, ioption)


if __name__ == "__main__":
    main()
